%% HDD Uncertainty Modeling

%% Load Data
% The file HDData.mat has the following variables:
%  wexp: Nw-by-1 vector of frequencies (rad/sec) for response data
%  RespData: Nw-by-Nexp matrix of experimental frequency response data.
%    The i^th column RespData(:,i) provides the experimental frequency
%    response data (at the frequencies in wexp) for the i^{th} system.
%  [A0,B0,C0,D0]: State space data for a nominal design fit This
%     was obtained by taking the mean of the 50 frequency responses and 
%     then using \texttt{fitfrd} (in Matlab) to fit a transfer function 
%     model to the average. The code for this fit is given below.
load('HDDdata');
[Nw,Nexp] = size(RespData);
G0 = ss(A0,B0,C0,D0);

% Create nominal by averaging data on a chosen window of frequencies
% wfit = wexp( wexp>0.3 & wexp<5 );
% GexpFIT = frd(freqresp(Gexp,wfit),wfit);
% GexpAVG = frd( sum(GexpFIT.ResponseData,4)/Nexp ,wfit);
% G0 = fitfrd(GexpAVG,4);


%% 1A) Bode Plots of Nominal and Data

% Create frequency response data (FRD) object
% (This allows the data to be manipulated like a normal state-space or
%  transfer function system object, e.g. we can draw Bode plots of FRDs)
Gexp = frd(RespData,wexp);

bopt = bodeoptions;
bopt.PhaseWrapping = 'on';

figure(1);
bodeplot(Gexp,'y',bopt); hold on;
ph = bodeplot(G0,'b',bopt); hold off;
xlim([wexp(1) wexp(end)]);
grid on;
if exist('garyfyFigure','file'); garyfyFigure; end

%% 1B) Relative Error

% Get frequency response of the Nominal Model
G0frd = frd(G0,wexp);    

% Compute the relative erro
for i=1:Nexp
    E(1,1,i) = abs( (G0frd-Gexp(:,:,i))/(G0frd) );
end

figure(2)
bodemag(E,'y')
xlim([wexp(1) wexp(end)]);
grid on;
if exist('garyfyFigure','file'); garyfyFigure; end

%% 1C) Uncertainty "Weight"
Wu = tf([1 1],[0.5 3])^4;

figure(3)
bodemag(E,'y',Wu,'b')
legend('E','Wu','Location','Best');
xlim([wexp(1) wexp(end)]);
grid on;
if exist('garyfyFigure','file'); garyfyFigure; end

